cmd/compile, runtime: use R20, R21 in ARM64's Duff's devices
Currently we use R16 and R17 for ARM64's Duff's devices.
According to ARM64 ABI, R16 and R17 can be used by the (external)
linker as scratch registers in trampolines. So don't use these
registers to pass information across functions.
It seems unlikely that calling Duff's devices would need a
trampoline in normal cases. But it could happen if the call
target is out of the 128 MB direct jump limit.
The choice of R20 and R21 is kind of arbitrary. The register
allocator allocates from low-numbered registers. High numbered
registers are chosen so it is unlikely to hold a live value and
forces a spill.
Fixes #32773.
Change-Id: Id22d555b5afeadd4efcf62797d1580d641c39218
Reviewed-on: https://go-review.googlesource.com/c/go/+/183842
Run-TryBot: Cherry Zhang <cherryyz@google.com>
Reviewed-by: Keith Randall <khr@golang.org>
diff --git a/src/runtime/mkduff.go b/src/runtime/mkduff.go
index b7c7e26..b6fe701 100644
--- a/src/runtime/mkduff.go
+++ b/src/runtime/mkduff.go
@@ -151,26 +151,26 @@
func zeroARM64(w io.Writer) {
// ZR: always zero
// R16 (aka REGRT1): ptr to memory to be zeroed
// On return, R16 points to the last zeroed dword.
fmt.Fprintln(w, "TEXT runtime·duffzero(SB), NOSPLIT|NOFRAME, $0-0")
for i := 0; i < 63; i++ {
- fmt.Fprintln(w, "\tSTP.P\t(ZR, ZR), 16(R16)")
+ fmt.Fprintln(w, "\tSTP.P\t(ZR, ZR), 16(R20)")
}
- fmt.Fprintln(w, "\tSTP\t(ZR, ZR), (R16)")
+ fmt.Fprintln(w, "\tSTP\t(ZR, ZR), (R20)")
fmt.Fprintln(w, "\tRET")
}
func copyARM64(w io.Writer) {
- // R16 (aka REGRT1): ptr to source memory
- // R17 (aka REGRT2): ptr to destination memory
+ // R20: ptr to source memory
+ // R21: ptr to destination memory
// R26, R27 (aka REGTMP): scratch space
- // R16 and R17 are updated as a side effect
+ // R20 and R21 are updated as a side effect
fmt.Fprintln(w, "TEXT runtime·duffcopy(SB), NOSPLIT|NOFRAME, $0-0")
for i := 0; i < 64; i++ {
- fmt.Fprintln(w, "\tLDP.P\t16(R16), (R26, R27)")
- fmt.Fprintln(w, "\tSTP.P\t(R26, R27), 16(R17)")
+ fmt.Fprintln(w, "\tLDP.P\t16(R20), (R26, R27)")
+ fmt.Fprintln(w, "\tSTP.P\t(R26, R27), 16(R21)")
fmt.Fprintln(w)
}
fmt.Fprintln(w, "\tRET")
@@ -151,26 +151,26 @@
func zeroARM64(w io.Writer) {
// ZR: always zero
- // R16 (aka REGRT1): ptr to memory to be zeroed
- // On return, R16 points to the last zeroed dword.
+ // R20: ptr to memory to be zeroed
+ // On return, R20 points to the last zeroed dword.
fmt.Fprintln(w, "TEXT runtime·duffzero(SB), NOSPLIT|NOFRAME, $0-0")
for i := 0; i < 63; i++ {